# Copyright 2013 Canonical Ltd.
# This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

import os

import sst.runtests

from u1testutils.sso import data

from acceptance import fixture_setup


class IdentityProviderSSTTestCase(sst.runtests.SSTTestCase):
    """Base test case for Identity Provider acceptance tests."""

    def setUp(self):
        # Set the base URL before calling the parent setUp, so it starts
        # the browser with this as SST's base url.
        self.base_url = os.environ['SST_BASE_URL']
        self.target_server = self._get_target_server()
        super(IdentityProviderSSTTestCase, self).setUp()

    def _get_target_server(self):
        """Return the name of the target server."""
        url = self.base_url.rstrip('/')
        if url in ('https://login.staging.ubuntu.com',
                   'https://login.staging.launchpad.net'):
            target = 'staging'
        elif url in ('https://login.ubuntu.com',
                     'https://login.launchpad.net'):
            target = 'production'
        else:
            target = 'devel'
        return target


class SSTTestCaseWithUserRegistered(IdentityProviderSSTTestCase):
    """Base test case for tests that require a registered user.

    On set up, this test case will register a user with the API.

    """

    def setUp(self):
        super(SSTTestCaseWithUserRegistered, self).setUp()
        self.user = get_new_test_user()
        self.useFixture(fixture_setup.UserRegistered(self.base_url, self.user))


def get_new_test_user(unique_id=None):
    return data.User.make_from_configuration(
        new_user=True, unique_id=unique_id)


class SSTTestCaseWithLogIn(SSTTestCaseWithUserRegistered):
    """Base test case for tests that require a logged in user.

    On set up, this test case will log in and leave the YourAccount page
    opened.

    """

    def setUp(self):
        super(SSTTestCaseWithLogIn, self).setUp()
        self.useFixture(fixture_setup.UserLoggedIn(self.user))
