# Copyright 2013 Canonical Ltd.
# This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

"""Set up and clean up fixtures for SSO acceptance tests."""

import fixtures

from django.core import management
from gargoyle import gargoyle
from u1testutils.sso import api

from acceptance import pages
from identityprovider import utils


class PreconditionsError(Exception):
    """Raised when the preconditions can not be set up."""
    pass


class UserRegistered(fixtures.Fixture):

    def __init__(self, base_url, user):
        super(UserRegistered, self).__init__()
        self.base_url = base_url
        self.user = user

    def setUp(self):
        super(UserRegistered, self).setUp()
        api_client = api.APIClient(self.base_url)
        # As we are using an email address that's whitelisted for the captcha
        # verification, it doesn't matter what values we send for it, as long
        # as we send some.
        error_message = 'Failed to register the user using the Ubuntu SSO API.'
        # On a development environment there's no need to delete the user
        # because it's easy and fast to reset the database. On staging and
        # production we have scripts that delete the users created by the
        # tests. So, no clean up is needed after creating the user.
        assert api_client.create_new_account(
            self.user, 'captcha id', 'captcha solution'), error_message


class UserLoggedIn(fixtures.Fixture):

    def __init__(self, user):
        super(UserLoggedIn, self).__init__()
        self.user = user

    def setUp(self):
        super(UserLoggedIn, self).setUp()
        log_in_page = pages.LogIn(open_page=True)
        log_in_page.log_in_to_site_recognized(self.user)


class BrandEnabled(fixtures.Fixture):

    def __init__(self, brand, target_server):
        super(BrandEnabled, self).__init__()
        self.brand = brand
        self.target_server = target_server

    def setUp(self):
        super(BrandEnabled, self).setUp()
        current_brand = utils.get_current_brand()
        if current_brand != self.brand:
            self._ensure_can_change_brand()
            self._change_brand()

    def _ensure_can_change_brand(self):
        if self.target_server != 'devel':
            raise PreconditionsError(
                'We can only change the brand on the devel server.')

    def _change_brand(self):
        # The clean up that enables the original brand fails like 50% of
        # the cases, leaving the new brand. I've reported bug 1188739,
        # and while we fix it, if we try to change the brand a
        # NotImplementedError will be thrown.
        raise NotImplementedError(
            'Current implementation is failing because of bug 1188739')
        if self.brand == 'ubuntuone':
            brand_switch = 'BRAND_UBUNTUONE'
        elif self.brand == 'launchpad':
            brand_switch = 'BRAND_LAUNCHPAD'
        else:
            raise NotImplementedError
        management.call_command(
            'add_switch', brand_switch, status=gargoyle.GLOBAL)
        self.addCleanup(
            management.call_command, 'remove_switch', brand_switch)
