# Copyright 2013 Canonical Ltd.
# This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

import re

from sst import actions
from u1testutils import mail
from u1testutils.sso import (
    api,
    mail as sso_mail
)

from acceptance import base, pages


class PasswordResetTestCase(base.SSTTestCaseWithUserRegistered):

    def setUp(self):
        super(PasswordResetTestCase, self).setUp()
        self._ensure_empty_inbox()
        api_client = api.APIClient(self.base_url)
        self.api_response = api_client.client.request_password_reset(
            self.user.email)

    def _ensure_empty_inbox(self):
        # As we are using a unique email address, the inbox will not have any
        # previous message for that address.
        # Wait for the account creation email to arrive.
        sso_mail.get_verification_link_for_address(self.user.email)
        # After getting the latest email, the helper deletes all the emails
        # sent to that email address.

    def test_token_contains_user_email_address(self):
        token = self.api_response.json()
        self.assertEqual(token['email'], self.user.email)

    def test_open_reset_password_link_from_email(self):
        link = self._get_password_reset_link()
        actions.go_to(link)
        pages.ResetPasswordStep3(open_page=False)

    def _get_password_reset_link(self):
        msg = mail.get_latest_email_sent_to(self.user.email).as_string()
        reset_link_re = (
            r'(http(?:s)?://[a-zA-Z0-9_\-\.]*(?::[0-9]+)?/'
            r'token/(?:.*)/\+resetpassword/.*)'
        )
        this_line = ''
        for line in msg.splitlines():
            if line.endswith('='):
                this_line += line[:-1]
                continue

            this_line += line
            match = re.match(reset_link_re, this_line)
            if match:
                break
            this_line = ''
        else:
            raise AssertionError('Password reset link not found')

        link = match.groups()[0]
        return link
