# Copyright 2012, 2013 Canonical Ltd.
# This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

import testscenarios

from u1testutils.sso import data

from acceptance import base, pages


class TestRequestTeams(
        testscenarios.TestWithScenarios, base.IdentityProviderSSTTestCase):

    scenarios = [
        ('one team', dict(
            requested_teams=['canonical-isd-hackers'],
            expected_teams=['canonical-isd-hackers'])),
        ('all teams', dict(
            requested_teams=['canonical-voices', 'canonical-isd-hackers'],
            expected_teams=['canonical-voices', 'canonical-isd-hackers'])),
        ('teams without membership', dict(
            requested_teams=[
                'canonical-voices', 'canonical-isd-hackers', 'canonical-isd',
                'canonical'],
            expected_teams=['canonical-voices', 'canonical-isd-hackers'])),
        ('non-existing team', dict(
            requested_teams=['non-existing-team'],
            expected_teams=[])),
    ]

    def setUp(self):
        super(TestRequestTeams, self).setUp()
        if self.target_server == 'production':
            self.skipTest("On production we can't access the test consumer.")
        elif self.target_server == 'staging':
            self.skipTest(
                "On staging, this test is not reliable. This is reported as "
                "bug #1169666.")
        self.user = data.User.make_from_configuration()

    def test_request_teams(self):
        consumer_page = pages.Consumer(open_page=True)
        log_in_page = consumer_page.begin(teams=self.requested_teams)
        site_not_recognized_page = log_in_page.log_in_to_site_not_recognized(
            self.user)
        site_not_recognized_page.make_all_information_available_to_website()
        site_not_recognized_page.yes_sign_me_in()
        consumer_finish_page = pages.ConsumerFinish()
        self.assertEqual(consumer_finish_page.get_teams(), self.expected_teams)
