# Copyright 2012, 2013 Canonical Ltd.
# This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

import sst.actions

from acceptance import base, pages
from acceptance.tests import devices


class TestDeviceLogin(base.SSTTestCaseWithLogIn):

    def setUp(self):
        super(TestDeviceLogin, self).setUp()
        self.fixture = self.useFixture(
            devices.TwoFactorPageOpenedWithTwoFactorRequired(
                ['2F Required Generic Device'], self.user))
        self.page = pages.TwoFactorVerify()

    def test_login_with_generic_device(self):
        one_time_pass = self.fixture.get_one_time_password(
            device_number=0, password_index=1)
        # Returns an YourAccount and automatically asserts it is open.
        self.page.two_factor_authentication(one_time_pass)

    def test_invalid_two_factor_password(self):
        two_factor_page = self.page.authenticate_2F_with_error('bad pass')
        one_time_password_error = two_factor_page.get_form_errors()
        self.assertEqual(
            one_time_password_error,
            'Please enter a 6-digit or 8-digit one-time password')

    def test_attempt_skipping_two_factor_prompt(self):
        # Should insist on 2F and not give access to '/'
        # In response to defect 923814
        sst.actions.go_to('/')
        self.assertEqual(self.page.qa_anchor, 'two_factor_verify')

    def test_account_suspended(self):
        # Enter an incorrect otp 18 times to trigger account suspension
        for attempts in range(17):
            self.page.authenticate_2F_with_error('123456')
        # The 18th time will return a YouAreSuspended page instead
        sst.actions.write_textfield('id_oath_token', '123456')
        auth_button = sst.actions.get_element_by_css(
            '*[data-qa-id="auth_button"]')
        sst.actions.click_button(auth_button)
        # The account is now suspended, the object will check this page exists
        pages.YouAreSuspended()
