# Copyright 2012, 2013 Canonical Ltd.
# This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

import testscenarios
from acceptance import base, pages
from acceptance.tests import devices


class TestRenameDevice(base.SSTTestCaseWithLogIn):

    def setUp(self):
        super(TestRenameDevice, self).setUp()
        self.useFixture(devices.RenameDevicePageOpened(
            'Original Device Name'))
        self.page = pages.RenameAuthenticationDevice()

    def test_device_rename(self):
        your_auth_devices = self.page.rename_device('The New Device Name')
        self.assertEqual(
            your_auth_devices.get_devices(), ['The New Device Name'])

    def test_cancel_device_rename(self):
        your_auth_devices = self.page.cancel()
        self.assertEqual(
            your_auth_devices.get_devices(), ['Original Device Name'])


class TestRenameDeviceWithErrors(
        testscenarios.TestWithScenarios, base.SSTTestCaseWithLogIn):

    scenarios = [
        ('empty string', dict(
            new_name='', error_msg='This field is required.')),
        ('single space', dict(
            new_name=' ', error_msg='The name must contain at least one '
            'non-whitespace character.')),
    ]

    def setUp(self):
        super(TestRenameDeviceWithErrors, self).setUp()
        self.useFixture(devices.RenameDevicePageOpened(
            'Original Device Name'))
        self.page = pages.RenameAuthenticationDevice()

    def test_rename_device_with_errors(self):
        rename_device = self.page.rename_device_with_errors(self.new_name)
        missing_name_error = rename_device.get_form_errors()
        self.assertEqual(missing_name_error, self.error_msg)
