# Copyright 2012, 2013 Canonical Ltd.
# This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

from acceptance import base, pages
from acceptance.tests import devices

import sst.actions


class TestDevicePreferences(base.SSTTestCaseWithLogIn):

    def test_device_preferences_hidden(self):
        your_account_page = pages.YourAccount(open_page=True)
        self.assertFalse(your_account_page.is_device_options_displayed())

    def test_device_preferences_visible_after_adding_device(self):
        self.useFixture(
            devices.AuthenticationDevicesPageOpenedWithDevicesAdded(
                ['Single device']))
        your_account_page = pages.YourAccount(open_page=True)
        self.assertTrue(your_account_page.is_device_options_displayed())

    def test_two_factor_default_is_unrequired(self):
        self.useFixture(
            devices.AuthenticationDevicesPageOpenedWithDevicesAdded(
                ['Single device']))
        pages.YourAccount(open_page=True)
        always_radio = sst.actions.get_element(
            tag="input", name="twofactor_required", value="True")
        as_needed_radio = sst.actions.get_element(
            tag="input", name="twofactor_required", value="False")
        sst.actions.assert_radio_value(always_radio, False)
        sst.actions.assert_radio_value(as_needed_radio, True)

    #   In response to defect #923808
    def test_two_factor_preferences_not_displayed_after_device_deletion(self):
        self.useFixture(devices.DeleteDevicesPageOpened(
            'Test device to delete'))
        self.page = pages.DeleteAuthenticationDevice()
        self.page.confirm_delete_device()
        your_account_page = pages.YourAccount(open_page=True)
        self.assertFalse(your_account_page.is_device_options_displayed())

    #   In response to defect #930379
    def test_two_factor_login_not_required_after_device_deletion(self):
        self.fixture = self.useFixture(
            devices.TwoFactorPageOpenedWithTwoFactorRequired(
                ['2F Required Generic Device'], self.user))
        self.page = pages.TwoFactorVerify()
        one_time_pass = self.fixture.get_one_time_password(
            device_number=0, password_index=1)
        self.page.two_factor_authentication(one_time_pass)
        your_auth_devices_page = pages.YourAuthenticationDevices(
            open_page=True)
        confirm_page = your_auth_devices_page.delete_authentication_device()
        confirm_page.confirm_delete_device()
        logout_landing_page = self.page.log_out()
        login_page = logout_landing_page.click_log_in_or_create_account()
        login_page.log_in_to_site_recognized(self.user)
        pages.YourAccount()

    def test_two_factor_login_not_required_for_as_needed_setting(self):
        self.useFixture(
            devices.AuthenticationDevicesPageOpenedWithDevicesAdded(
                ['Single device']))
        self.page = pages.YourAuthenticationDevices()
        logout_landing_page = self.page.log_out()
        login_page = logout_landing_page.click_log_in_or_create_account()
        login_page.log_in_to_site_recognized(self.user)
        pages.YourAccount()
