# Copyright 2013 Canonical Ltd.
# This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

import sst.actions

from acceptance import base, pages
from acceptance.tests import devices


class TestTwoFactorSynchronization(base.SSTTestCaseWithLogIn):

    def setUp(self):
        super(TestTwoFactorSynchronization, self).setUp()
        self.fixture = self.useFixture(
            devices.TwoFactorPageOpenedWithTwoFactorRequired(user=self.user))
        self.page = pages.TwoFactorVerify()

    def token_for(self, counter):
        return self.fixture.get_one_time_password(
            device_number=0, password_index=counter)

    def test_starts_without_sync_link(self):
        self.assertIsNone(self.page.get_sync_link())

    def test_shows_sync_link_after_failure(self):
        self.page.authenticate_2F_with_error('111111')
        self.assertIsNotNone(self.page.get_sync_link())

    def test_synchronizes_a_device_after_login_failure(self):
        self.page.authenticate_2F_with_error('111111')
        sync_page = self.page.click_sync_link()

        token_fields = sync_page.get_token_fields()
        # Initial counter is 0, so starting from 5 is distant enough
        counter = 5
        for field in token_fields:
            sync_page.write_token(field, self.token_for(counter))
            counter += 1

        sync_page.synchronize()

        expected_url = sst.actions.get_base_url() + pages.YourAccount.url_path
        sst.actions.assert_url(expected_url)
