#! /usr/bin/env python

import sys
import os

PATHS = [
    # src and config
    '.',
    'src',
    # dependencies
    'lib',
]

CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))
PROJECT_ROOT_DIR = os.path.abspath(os.path.join(CURRENT_DIR, os.pardir))

DEFAULT_LOCAL_CFG_DIR = os.path.abspath(
    os.path.join(PROJECT_ROOT_DIR, os.pardir, 'local_config'))
DEFAULT_LOCAL_CFG_PATH = os.path.join(DEFAULT_LOCAL_CFG_DIR, 'local.cfg')

LOCAL_CFG_PATH = os.path.abspath(
    os.environ.get('CONFIGGLUE_LOCAL_CONFIG', DEFAULT_LOCAL_CFG_PATH))
LOCAL_CFG_DIR = os.path.dirname(LOCAL_CFG_PATH)


def get_paths(paths):
    """Sets up necessary python paths for Pay in prod/staging"""
    # only include a path if not already in sys.path to avoid duplication of
    # paths when using code reloading
    path_set = set(sys.path)
    for p in paths:
        path = os.path.abspath(os.path.join(PROJECT_ROOT_DIR, p))
        if path not in path_set:
             yield path


def setup_paths():
    sys.path = list(get_paths(PATHS)) + sys.path
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'django_project.settings')


if __name__ == '__main__':
    # For use in shell scripting
    # e.g. $(python paths.py)
    print "export PYTHONPATH=%s" % ":".join(get_paths(PATHS))
    print "export DJANGO_SETTINGS_MODULE=django_project.settings"
