import os
import sys

# before doing anything else, patch stdout to avoid breakage in production
# see LP: #1099459 for details
sys.stdout = sys.stderr

# add current dir to system path if not already there
# check if path is already there to avoid path duplication when
# modwsgi is configured for code reloading
curdir = os.path.abspath(os.path.dirname(__file__))
if curdir not in sys.path:
    sys.path.append(curdir)
import paths
paths.setup_paths()

from django.conf import settings

os.environ['PGCONNECT_TIMEOUT'] = str(settings.PGCONNECT_TIMEOUT)
os.environ['PGSSLMODE'] = 'allow'

from identityprovider.wsgi_handler import OOPSWSGIHandler

app = OOPSWSGIHandler()

# Wrap the application in the Oops wsgi app to catch unhandled exceptions
# and create oops for them.

from identityprovider.oops_config import global_error_reporter
from oops_wsgi import make_app
from timeline_django import setup as timeline_django_setup
from timeline_django.wsgi import make_app as timeline_django_make_app
from timeline.wsgi import make_app as timeline_make_app

# First we create the config that defines what to do with the oopses.
config = global_error_reporter.setup()

# Install the hooks to record the sql queries
timeline_django_setup.setup_for_requests()

# Set up the timeline in the django and wsgi envs
app = timeline_django_make_app(app)
app = timeline_make_app(app)

# Then we wrap the django app in the oops one
application = make_app(app, config, oops_on_status=['500'],
                       soft_start_timeout=settings.HANDLER_TIMEOUT_MILLIS)
