# Copyright 2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import

import os
import sys

from fabric.context_managers import lcd

from .constants import I18N_APPS
from .environment import virtualenv_local


def get_django_settings(*keys):
    # setup the correct django environment
    sys.path.insert(0, os.path.abspath(os.curdir))
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'django_project.settings')
    # get django settings
    from django.conf import settings

    result = dict.fromkeys(keys)
    for key in keys:
        result[key] = getattr(settings, key)

    return result


def brand(brand):
    os.environ["CONFIGGLUE_BRANDING_BRAND"] = brand


def manage(command, *args, **kwargs):
    """Run manage.py command"""
    relpath = kwargs.pop('relpath', '.')
    cmd = [
        'python',
        "%s/django_project/manage.py" % relpath,
        command,
    ]
    args += tuple("%s=%s" % (k, v) for (k, v) in kwargs.items())
    cmd.extend(args)

    virtualenv_local(" ".join(cmd), capture=False)


def syncdb():
    """Sync the database.

    If south is listed in INSTALLED_APPS syncdb runs with --migrate.
    """
    # avoid circular import loop
    from .database import patch_database_settings

    args = ['--noinput']
    django_settings = get_django_settings('INSTALLED_APPS')
    if 'south' in django_settings['INSTALLED_APPS']:
        args.append('--migrate')

    with patch_database_settings(USER='postgres', PASSWORD=''):
        manage('syncdb', *args)


def makemessages():
    """Create/Update translation strings in .po files."""
    django_settings = get_django_settings('SUPPORTED_LANGUAGES')
    supported_languages = django_settings['SUPPORTED_LANGUAGES']
    for app in I18N_APPS:
        for lang in supported_languages:
            with lcd("src/{0}".format(app)):
                if '-' in lang:
                    language, country = lang.split('-')
                    locale_name = "{0}_{1}".format(language, country.upper())
                else:
                    locale_name = lang
                manage('makemessages -l {0} -e .html,.txt'.format(locale_name),
                       relpath='../..')


def compilemessages(*args):
    """Compile .po translation files into binary (.mo)."""
    for app in I18N_APPS:
        with lcd("src/{0}".format(app)):
            manage('compilemessages ' + ' '.join(args), relpath='../..')
