# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from django.conf.urls import patterns, url

from piston.authentication import HttpBasicAuthentication
from piston.resource import Resource

from identityprovider.auth import (
    basic_authenticate,
    SSOOAuthAuthentication,
)

import api.v10.handlers as v10
import api.v11.handlers as v11
import api.v20.handlers as v20

from api.v20.auth import (
    ApiAccountsAuthentication,
    ApiEmailsAuthentication,
    ApiTokensAuthentication,
    ApiResource,
)

v10root = Resource(handler=v10.RootHandler)
v10captcha = Resource(handler=v10.CaptchaHandler)
v10registration = Resource(handler=v10.RegistrationHandler)
v10auth = Resource(
    handler=v10.AuthenticationHandler,
    authentication=HttpBasicAuthentication(auth_func=basic_authenticate)
)
v10accounts = Resource(
    handler=v10.AccountsHandler,
    authentication=SSOOAuthAuthentication()
)


v11root = Resource(handler=v11.RootHandler)
v11auth = Resource(
    handler=v11.AuthenticationHandler,
    authentication=HttpBasicAuthentication(auth_func=basic_authenticate)
)


v2accounts = ApiResource(
    handler=v20.AccountsHandler, authentication=ApiAccountsAuthentication())
v2emails = ApiResource(
    handler=v20.EmailsHandler, authentication=ApiEmailsAuthentication())
v2login = ApiResource(handler=v20.AccountLoginHandler)
v2login_phone = ApiResource(handler=v20.AccountPhoneLoginHandler)
v2password_reset = ApiResource(handler=v20.PasswordResetTokenHandler)
v2registration = ApiResource(handler=v20.AccountRegistrationHandler)
v2requests = ApiResource(handler=v20.RequestsHandler)
v2tokens = ApiResource(
    handler=v20.TokensHandler, authentication=ApiTokensAuthentication())


urlpatterns = patterns(
    '',
    # v1.0
    url(r'^1.0/$', v10root, name='api-10-root',
        kwargs={'emitter_format': 'lazr.restful'}),
    url(r'^1.0/captchas$', v10captcha, name='api-10-captchas',
        kwargs={'emitter_format': 'lazr.restful'}),
    url(r'^1.0/registration$', v10registration, name='api-10-registration',
        kwargs={'emitter_format': 'lazr.restful'}),
    url(r'^1.0/authentications$', v10auth, name='api-10-authentications',
        kwargs={'emitter_format': 'lazr.restful'}),
    url(r'^1.0/accounts$', v10accounts, name='api-10-accounts',
        kwargs={'emitter_format': 'lazr.restful'}),

    # v1.1
    url(r'^1.1/$', v11root,
        kwargs={'emitter_format': 'lazr.restful'}),
    # add backwards compatible endpoints
    url(r'^1.1/captchas$', v10captcha,
        kwargs={'emitter_format': 'lazr.restful'}),
    url(r'^1.1/registration$', v10registration,
        kwargs={'emitter_format': 'lazr.restful'}),
    url(r'^1.1/accounts$', v10accounts,
        kwargs={'emitter_format': 'lazr.restful'}),
    # add overriding endpoints
    url(r'^1.1/authentications$', v11auth,
        kwargs={'emitter_format': 'lazr.restful'}),

    # v2
    url(r'^v2/accounts$', v2registration, name='api-registration'),
    url(r'^v2/accounts/(?P<openid>\w+)$', v2accounts, name='api-account'),
    # emails temporarily hooked up so we can do reverse()
    url(r'^v2/emails/(?P<email>.+)$', v2emails, name='api-email'),
    url(r'^v2/requests/validate$', v2requests, name='api-requests'),
    url(r'^v2/tokens/oauth$', v2login, name='api-login'),
    url(r'^v2/tokens/oauth/(?P<token>.+)$', v2tokens, name='api-token'),
    url(r'^v2/tokens/password$', v2password_reset, name='api-password-reset'),
    # login from phone, with a phone user id
    url(r'^v2/tokens/phone$', v2login_phone, name='api-login-phone'),
)
