# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
import re

from django.conf import settings
from django.http import HttpResponse
from django_statsd.clients import statsd
from piston.decorator import decorator
from piston.utils import throttle as piston_throttle

from api.v20.utils import errors


def throttle(max_requests, timeout=60*60, extra='', extra_callback=None):
    def is_whitelisted(request):
        preferredemail = getattr(request.user, 'preferredemail', None)
        if preferredemail is None:
            return False

        for pattern in settings.THROTTLE_WHITELIST_EMAIL_REGEXP_LIST:
            if re.match(pattern, preferredemail.email):
                return True
        return False

    @decorator
    def wrap(f, self, request, *args, **kwargs):
        if is_whitelisted(request):
            return f(self, request, *args, **kwargs)

        # apply throttling checks
        if extra_callback is not None:
            local_extra = extra_callback(request, *args, **kwargs)
        else:
            local_extra = extra
        response = piston_throttle(
            max_requests, timeout=timeout, extra=local_extra)(f)(
                self, request, *args, **kwargs)
        if (isinstance(response, HttpResponse)
                and response.status_code == 503):
            statsd.incr('throttling.throttled')
            f_name = ".".join([
                self.__class__.__module__,
                self.__class__.__name__,
                f.__name__,
            ])
            statsd.incr('throttling.' + f_name + '.throttled')
            error = errors.TOO_MANY_REQUESTS(**{
                'Retry-After': response['Retry-After']})
            response = error
        return response
    return wrap
