# NOTE: Once we switch to Django 1.6, this will no longer be
# necessary, as that version provides a CSRF_COOKIE_HTTPONLY setting for
# this very purpose.

from django.conf import settings
from django.middleware import csrf as django_csrf


class CsrfViewMiddleware(django_csrf.CsrfViewMiddleware):

    def process_response(self, request, response):
        response = super(CsrfViewMiddleware, self).process_response(
            request, response)
        if settings.CSRF_COOKIE_NAME in response.cookies:
            # Set the CSRF cookie even if it's already set, so we renew
            # the expiry timer and set the httponly bit as needed.
            response.set_cookie(settings.CSRF_COOKIE_NAME,
                                request.META["CSRF_COOKIE"],
                                max_age=60 * 60 * 24 * 7 * 52,
                                domain=settings.CSRF_COOKIE_DOMAIN,
                                path=settings.CSRF_COOKIE_PATH,
                                secure=settings.CSRF_COOKIE_SECURE,
                                httponly=settings.CSRF_COOKIE_HTTPONLY,
                                )
        return response
