# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from django.db import models

from identityprovider.const import PERSON_VISIBILITY_PUBLIC
from identityprovider.models import Person

__all__ = (
    'get_team_memberships_for_user',
    'TeamParticipation',
)


def get_team_memberships_for_user(team_names, user, include_private=False):
    if user.person is None:
        # Someone that has no person associated with the account can't
        # be a member of any teams.
        return []
    teams = TeamParticipation.objects.filter(
        person=user.person,
        team__name__in=team_names,
    )
    if not include_private:
        teams = teams.filter(
            team__visibility=PERSON_VISIBILITY_PUBLIC)
    # By default it's a query set result, not a true list
    return list(teams.values_list('team__name', flat=True))


class TeamParticipation(models.Model):
    team = models.ForeignKey(Person, db_column='team', null=True,
                             related_name='team_participations')
    person = models.ForeignKey(Person, db_column='person', null=True)

    class Meta:
        app_label = 'identityprovider'
        db_table = u'lp_teamparticipation'
        unique_together = ('team', 'person')

    def __unicode__(self):
        return "%s in %s" % (unicode(self.person), unicode(self.team))
