from django.db.backends.dummy.base import (
    DatabaseWrapper as BaseDatabaseWrapper,
    ignore,
)

__all__ = [
    'DatabaseWrapper',
]


class NopCursorWrapper(object):
    """
    This cursor wrappper ignores any operations used on it.
    """
    execute = ignore
    executemany = ignore
    callproc = ignore
    fetchone = ignore
    rowcount = 0
    lastrowid = None

    def fetchmany(self, *args):
        return []

    def fetchall(self):
        return []


class DatabaseWrapper(BaseDatabaseWrapper):
    """Backend which ignores any SQL send to it."""

    # define a valid set of operators
    operators = {
        'exact': '= %s',
        'iexact': '= UPPER(%s)',
        'contains': 'LIKE %s',
        'icontains': 'LIKE UPPER(%s)',
        'regex': '~ %s',
        'iregex': '~* %s',
        'gt': '> %s',
        'gte': '>= %s',
        'lt': '< %s',
        'lte': '<= %s',
        'startswith': 'LIKE %s',
        'endswith': 'LIKE %s',
        'istartswith': 'LIKE UPPER(%s)',
        'iendswith': 'LIKE UPPER(%s)',
    }

    # Override the base class implementations with null
    # implementations.
    _commit = ignore
    enter_transaction_management = ignore
    set_clean = ignore
    commit_unless_managed = ignore
    savepoint_commit = ignore

    def __init__(self, *args, **kwargs):
        super(DatabaseWrapper, self).__init__(*args, **kwargs)

        self.features.supports_transactions = False
        self.ops.quote_name = lambda x: x

    def cursor(self):
        return NopCursorWrapper()
