from datetime import datetime

from django.core.management import call_command

from identityprovider.management.commands import clean_sessions
from identityprovider.tests.utils import SSOBaseTestCase


class CleanSessionsCommandTestCase(SSOBaseTestCase):

    def setUp(self):
        super(CleanSessionsCommandTestCase, self).setUp()
        self._apply_patch(
            'identityprovider.management.commands.clean_sessions.'
            'clean_sessions_until', return_value=None)

    def test_clean_sessions_without_limit(self):
        date_string = '2013-10-24'
        date_time = datetime.strptime(date_string, '%Y-%m-%d')

        call_command('clean_sessions', creation_date=date_string)

        clean_sessions.clean_sessions_until.assert_called_once_with(
            date_time, limit=None, dry_run=False)

    def test_clean_sessions_with_limit(self):
        date_string = '2013-10-24'
        date_time = datetime.strptime(date_string, '%Y-%m-%d')

        call_command('clean_sessions', creation_date=date_string, limit=10)

        clean_sessions.clean_sessions_until.assert_called_once_with(
            date_time, limit=10, dry_run=False)

    def test_clean_sessions_with_dry_run(self):
        date_string = '2013-10-24'
        date_time = datetime.strptime(date_string, '%Y-%m-%d')

        call_command('clean_sessions', creation_date=date_string, dry_run=True)

        clean_sessions.clean_sessions_until.assert_called_once_with(
            date_time, limit=None, dry_run=True)
